import { createSlice, PayloadAction } from '@reduxjs/toolkit';
import type { RootState } from './store';

// Define the initial state using that type
export const initialState: {
  data: {
    nodes: Record<string, any>[];
    edges: Record<string, any>[];
  };
  schema: {
    nodes: Record<string, any>[];
    edges: Record<string, any>[];
  };
  querybuilder: {
    nodes: Record<string, any>[];
    edges: Record<string, any>[];
  };
} = {
  data: {
    nodes: [],
    edges: [],
  },
  schema: {
    nodes: [],
    edges: [],
  },
  querybuilder: {
    nodes: [],
    edges: [],
  },
};

export const searchResultSlice = createSlice({
  name: 'search',
  initialState,
  reducers: {
    addSearchResultData: (state, action: PayloadAction<{ nodes: Record<string, any>[]; edges: Record<string, any>[] }>) => {
      state.data = action.payload;
    },
    addSearchResultSchema: (state, action: PayloadAction<{ nodes: Record<string, any>[]; edges: Record<string, any>[] }>) => {
      state.schema = action.payload;
    },
    addSearchResultQueryBuilder: (state, action: PayloadAction<{ nodes: Record<string, any>[]; edges: Record<string, any>[] }>) => {
      state.querybuilder = action.payload;
    },
    addSearchResultSelected: (state, action: PayloadAction<{ category: string; value: Record<string, any> }>) => {
      state.data = { nodes: [], edges: [] };
      state.schema = { nodes: [], edges: [] };
      state.querybuilder = { nodes: [], edges: [] };

      const { category, value } = action.payload;
      state[category] = value;
    },
    resetSearchResults: (state) => {
      return {
        ...state,
        data: initialState.data,
        schema: initialState.schema,
        querybuilder: initialState.querybuilder,
      };
    },
  },
});

export const { addSearchResultData, addSearchResultSchema, addSearchResultQueryBuilder, addSearchResultSelected, resetSearchResults } =
  searchResultSlice.actions;

// Other code such as selectors can use the imported `RootState` type
export const searchResultState = (state: RootState) => state.searchResults;
export const searchResultData = (state: RootState) => state.searchResults.data;
export const searchResultSchema = (state: RootState) => {
  const nodes = state.searchResults.schema.nodes.map((node) => node.key);
  const edges = state.searchResults.schema.edges.map((edge) => edge.key);
  return [...nodes, ...edges];
};
export const searchResultQB = (state: RootState) => state.searchResults.querybuilder;

export default searchResultSlice.reducer;
